import os.path
from remin_service.helper import DataCent
import threading
import yaml


def merge_dicts(*dicts):
    result = {}
    for d in dicts:
        for key, value in d.items():
            if isinstance(value, dict) and key in result:
                result[key] = merge_dicts(result[key], value)
            else:
                result[key] = value
    return result


class ConfigLoad:

    _instance_lock = threading.Lock()

    @classmethod
    def instance(cls, config_file):
        with ConfigLoad._instance_lock:
            if not hasattr(ConfigLoad, "_instance"):
                ConfigLoad._instance = ConfigLoad(config_file)
        return ConfigLoad._instance

    def __init__(self, config_file):
        self.config_file = config_file

    def load(self):

        with open(self.config_file, encoding="utf8") as file:

            DataCent.data = merge_dicts(DataCent.data, yaml.load(file.read(), Loader=yaml.FullLoader))
            # DataCent.data.update(
            #     yaml.load(file.read(), Loader=yaml.FullLoader)
            # )

        active = DataCent.data.get("config", {}).get("active")
        if not active:
            return True

        file_s = self.config_file.split(".")

        new_config_file = file_s[0] + f".{active}." + file_s[1]
        if not os.path.exists(new_config_file):
            raise FileNotFoundError(f"{new_config_file} 不存在")

        with open(new_config_file, encoding="utf8") as file:

            DataCent.data = merge_dicts(DataCent.data, yaml.load(file.read(), Loader=yaml.FullLoader))
            # DataCent.data.update(
            #     yaml.load(file.read(), Loader=yaml.FullLoader)
            # )
        # print(DataCent.data)

        return True


if __name__ == '__main__':
    ConfigLoad.instance("/src/template/resources/config.yaml").load()


from pydantic import BaseModel, field_validator
from remin_service.helper import DataCent
from typing import Optional
from remin_service.helper.ip_helper import get_local_ip, get_public_ip

# data = DataCent.data


class ImportConfig:

    def __init__(self, config_key):
        self._config_key = config_key

    def __call__(self, cls):
        _config_keys = self._config_key.split(".")
        # print(_config_keys)
        data = DataCent.data
        for _config_key in _config_keys:
            # print(data)
            data = data.get(_config_key, {})

        return cls(**data)


@ImportConfig("config")
class Config(BaseModel):
    active: str


@ImportConfig("service")
class ServiceConfig(BaseModel):
    ip: str = ""
    ip_type: str = "local"
    port: int = 8080
    service_name: str = "service"

    def __init__(self, **kwargs):
        if "ip" not in kwargs:
            kwargs["ip"] = "0.0.0.0"
            kwargs["ip_type"] = "public"
        elif kwargs.get("ip") == "0.0.0.0":
            kwargs["ip_type"] = "local_public"
        super().__init__(**kwargs)

    @classmethod
    def get_ip(cls):
        if ServiceConfig.ip_type == "local_public":
            return get_local_ip()
        if ServiceConfig.ip_type == "local":
            return "127.0.0.1"
        if ServiceConfig.ip_type == "public":
            return get_public_ip()


@ImportConfig("cloud.nacos")
class NaNosConfig(BaseModel):
    class __Discovery(BaseModel):
        username: str = None
        password: str = None
        cluster_name: str = ""
        namespace: str = ""
        group: str = ""
        server_addr: str

    discovery: Optional[__Discovery] = None








import asyncio

from remin_service.log import logger
import os
import threading
from fastapi.staticfiles import StaticFiles
from remin_service.base.controller import Controller
from remin_service.base.base_app import RemitFastApi
from remin_service.base.base_router import BaseRemitAPIRouter
from remin_service.helper.config_helper import ConfigLoad


class FastSkeletonApp:

    _instance_lock = threading.Lock()
    app = None
    init_routes = None

    @classmethod
    def instance(cls, init_routes, title="Et Admin API接口文档管理系统"):
        with FastSkeletonApp._instance_lock:
            if not hasattr(FastSkeletonApp, "_instance"):
                FastSkeletonApp._instance = FastSkeletonApp(init_routes, title)

        return FastSkeletonApp._instance

    def __init__(
            self,
            init_routes,
            base_path=".",
            config_path=None,
            static_dir=f"{os.path.dirname(__file__)}/static",
            middleware_func=None
    ):
        if not config_path:
            config_path = base_path
        config_file = f"{config_path}/resources/config.yaml"
        ConfigLoad.instance(config_file).load()
        self.app = RemitFastApi(
            docs_url=None,
            redoc_url=None,
            api_router_class=BaseRemitAPIRouter,
            title="Api接口",
            version='1.0'
        )
        from .config import Config, ServiceConfig, NaNosConfig

        self.nacos_client = None
        if NaNosConfig.discovery and NaNosConfig.discovery.server_addr:
            from nacos import NacosClient
            self.nacos_client = NacosClient(
                server_addresses=NaNosConfig.discovery.server_addr,
                namespace=NaNosConfig.discovery.namespace
            )

        self.config = Config
        self.app.HOST = ServiceConfig.ip
        self.app.PORT = ServiceConfig.port
        self.app.ServerName = ServiceConfig.service_name
        self.app.AppBaseDir = base_path
        self.static_dir = static_dir
        self.middleware_func = middleware_func

        # 路由配置
        self.init_routes = init_routes
        Controller.init_app(self.app)
        self.init()

    def init(self):

        self.app.mount(
            '/static',
            StaticFiles(directory=self.static_dir),
            name='static'
        )
        from .config import NaNosConfig, ServiceConfig

        if NaNosConfig.discovery and NaNosConfig.discovery.server_addr:
            self.nacos_client.add_naming_instance(
                ServiceConfig.service_name, ServiceConfig.get_ip(), ServiceConfig.port,
                cluster_name=NaNosConfig.discovery.cluster_name,
                group_name=NaNosConfig.discovery.group
            )
            logger.info("Nacos 注册成功...")

            async def long_running_task():
                """一个长时间运行的异步任务示例"""
                while True:
                    self.nacos_client.send_heartbeat(
                        service_name=ServiceConfig.service_name,
                        ip=ServiceConfig.get_ip(),
                        port=ServiceConfig.port,
                        cluster_name=NaNosConfig.discovery.cluster_name,
                        group_name=NaNosConfig.discovery.group
                    )
                    await asyncio.sleep(10)  # 每30秒执行一次

            @self.app.on_event("startup")
            async def startup_event():
                logger.info("启动长时间运行的任务...")
                asyncio.create_task(long_running_task())

            @self.app.on_event("shutdown")
            async def shutdown_event():
                self.nacos_client.remove_naming_instance(
                    ServiceConfig.service_name, ServiceConfig.get_ip(), ServiceConfig.port,
                    cluster_name=NaNosConfig.discovery.cluster_name,
                    group_name=NaNosConfig.discovery.group
                )

        # 批量导入注册路由
        self.init_routes(self.app)

        # 注册中间件
        if self.middleware_func:
            self.middleware_func(self.app)

        @self.app.on_event("startup")
        async def startup_event():
            logger.info(f"[Api] 【{self.app.ServerName}】-【{self.config.active}】服务启动")

        @self.app.on_event("shutdown")
        async def shutdown_event():
            logger.info(f"[Api] 【{self.app.ServerName}】-【{self.config.active}服务关闭")





from setuptools import setup, find_packages


setup(
    name="remit_service",
    version="1.1",
    packages=find_packages(
        include=[
            "remin_service", "remin_service.*"
        ]
        # exclude=[
        #     "src",
        # ]
    ),
    package_data={
        'remin_service': ['static/**'],  # 包含所有 .py 文件
    },
    install_requires=[
        "fastapi==0.115.6",
        "uvicorn==0.34.0",
        "loguru==0.7.3",
        "PyYAML==6.0.2",
        "PyYAML==6.0.2",
        "python-nacos==0.1.1"
    ],
    python_requires=">=3.10",
    include_package_data=True
)